data {
  // observations
  int<lower=0> N;             // number of observations
  int<lower=0> C;             
  matrix[N,C] ZB;                // predictor for observation n
  vector[N] M;                // outcome for observation n
}
parameters {
  real<lower=0> sigma;        // noise variance
  real<lower=0> sigma_beta;        // smoothing parameter.
  vector[C] beta;           // regression  
}
transformed parameters{
    vector[N] mu;
    mu <- ZB * beta;
}
model {
  // priors
  sigma ~ cauchy(0,5);
  sigma_beta ~ cauchy(0,5);
  beta[1] ~ normal(0,2*sigma);
  for (l in 2:C)
   beta[l] ~ normal(beta[l-1],sigma_beta);

  // likelihood
  M ~ normal(mu, sigma);
}

